@@ -198,11 +198,6 @@ static void ap_selection_changed(GtkWidget* selection, gpointer data)
                                     GTK_TREE_VIEW(W(commandListView)));
     AccessPoint_s* ap;
 
-    if (W(cmd.changed)) {
-        W(changed) = TRUE;
-        update_ap_model_commands(app_data, W(ap.currentIter));
-    }
-
     /* Use-Case: all APs are deleted */
     if (!gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection),
                                                          &model, &iter)) {
@@ -212,8 +207,7 @@ static void ap_selection_changed(GtkWidget* selection, gpointer data)
     }
     enable_ap_general_toolbuttons(app_data, TRUE);
     gtk_widget_set_sensitive(GTK_WIDGET(W(cmd.addToolButton)), TRUE);
-   
- 
+
 //    enable_cmd_toolbuttons(app_data, TRUE);
 //    gtk_widget_set_sensitive(gtk_widget(w(cmd.addtoolbutton)), true);
 //    enable_ap_general_toolbuttons(app_data, TRUE);
@@ -230,7 +224,6 @@ static void ap_selection_changed(GtkWidget* selection, gpointer data)
     // TODO: what about g_object_unref to the model?
 
     W(ap.currentIter) = gtk_tree_iter_copy(&iter);
-    W(cmd.changed) = FALSE;
 }
 
 // TODO: copy&paste of 'ap_disabled_state_changed()"
@@ -240,7 +233,8 @@ static void cmd_disabled_state_changed(GtkCellRenderer* cell_renderer,
     GET_APP_DATA(data);
     inverse_cmd_disabled_field(path_string, data, NULL);
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 static void cmd_superuser_env_state_changed(GtkCellRenderer* cell_renderer,
@@ -249,7 +243,8 @@ static void cmd_superuser_env_state_changed(GtkCellRenderer* cell_renderer,
     GET_APP_DATA(data);
     inverse_cmd_superuser_env_field(path_string, data, NULL);
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 static void cmd_selection_changed(GtkWidget* selection, gpointer data)
@@ -300,7 +295,8 @@ static void cmdline_changed(GtkCellRenderer* cell_renderer, gchar* path_string,
     free_command(cmd);
     gtk_tree_path_free(path);
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 
@@ -439,34 +435,39 @@ static void add_cmd_toolbutton_clicked(GtkWidget* widget, gpointer data)
 {
     GET_APP_DATA(data);
 
-    GtkTreeModel* model;
+    GtkTreeModel* ap_model = gtk_tree_view_get_model(GTK_TREE_VIEW(
+							  W(apListView)));
+    GtkTreeModel* cmd_model;
+
     GtkTreeIter iter;
     Command_s* cmd;
     gboolean to_append = FALSE;
 
-    if (!show_add_command_dialog(GTK_WINDOW(W(mainWindow)), FALSE, &cmd))
+    if (!show_add_command_dialog(GTK_WINDOW(W(mainWindow)), ap_model,
+				 FALSE, &cmd))
 	return;
 
     if (!get_current_tree_iter(GTK_TREE_VIEW(W(commandListView)),
-			       &model, &iter))
+			       &cmd_model, &iter))
 	to_append = TRUE;
 
     if (to_append)
-	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
+	gtk_list_store_append(GTK_LIST_STORE(cmd_model), &iter);
     else
     {
 	GtkTreeIter new_iter;
 
-	gtk_list_store_insert_after(GTK_LIST_STORE(model), &new_iter, &iter);
+	gtk_list_store_insert_after(GTK_LIST_STORE(cmd_model), &new_iter, &iter);
 	iter = new_iter;
     }
 
-    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, cmd, -1);
+    gtk_list_store_set(GTK_LIST_STORE(cmd_model), &iter, 0, cmd, -1);
     select_treeview_row(W(commandListView), &iter);
 
     free_command(cmd);
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 // TODO: remove copy&paste; similar code 'edit_ap_name_toolbutton_clicked'
@@ -474,24 +475,29 @@ static void edit_cmd_toolbutton_clicked(GtkWidget* widget, gpointer data)
 {
     GET_APP_DATA(data);
 
-    GtkTreeModel* model;
+    GtkTreeModel* ap_model = gtk_tree_view_get_model(GTK_TREE_VIEW(
+							  W(apListView)));
+    GtkTreeModel* cmd_model;
     GtkTreeIter iter;
     Command_s* cmd;
 
     if (!get_current_tree_iter(GTK_TREE_VIEW(W(commandListView)),
-			       &model, &iter))
+			       &cmd_model, &iter))
 	ccreturn("Can't get current iter!");
 
-    gtk_tree_model_get(model, &iter, 0, &cmd, -1);
-    if (!show_add_command_dialog(GTK_WINDOW(W(mainWindow)), TRUE, &cmd)) {
+    gtk_tree_model_get(cmd_model, &iter, 0, &cmd, -1);
+    if (!show_add_command_dialog(GTK_WINDOW(W(mainWindow)), ap_model,
+				 TRUE, &cmd)) {
 	free_command(cmd);
 	return;
     }
 
-    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, cmd, -1);
+    gtk_list_store_set(GTK_LIST_STORE(cmd_model), &iter, 0, cmd, -1);
 
     free_command(cmd);
-    W(cmd.changed) = TRUE;
+
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 // TODO: remove copy&paste; similar code in 'del_ap_toolbutton_clicked'
@@ -518,7 +524,8 @@ static void del_cmd_toolbutton_clicked(GtkWidget* widget, gpointer data)
         gtk_tree_iter_free(next_iter);
     }
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 // TODO: don't like this implementation (w/a). should be revised
@@ -538,7 +545,8 @@ static void move_up_cmd_toolbutton_clicked(GtkWidget* widget, gpointer data)
     (void)swap_list_store_items(model, &iter, TRUE);
     cmd_selection_changed((GtkWidget *)selection, data);
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 static void move_down_cmd_toolbutton_clicked(GtkWidget* widget, gpointer data)
@@ -557,7 +565,8 @@ static void move_down_cmd_toolbutton_clicked(GtkWidget* widget, gpointer data)
     (void)swap_list_store_items(model, &iter, FALSE);
     cmd_selection_changed((GtkWidget *)selection, data);
 
-    W(cmd.changed) = TRUE;
+    update_ap_model_commands(app_data, W(ap.currentIter));
+    W(changed) = TRUE;
 }
 
 #ifdef MAEMO
@@ -599,7 +608,7 @@ static void close_application(GtkWidget* widget, GdkEvent* event, gpointer data)
 {
     GET_APP_DATA(data);
 
-    if (W(changed) || W(cmd.changed)) {
+    if (W(changed)) {
         // TODO: make Ok, No, Cancel buttons
         gint response;
         GtkWidget* confirm_save_dialog = gtk_message_dialog_new(
@@ -628,18 +637,9 @@ static void apply_changes(GtkWidget* widget, gpointer data)
     GtkTreeModel* ap_model = gtk_tree_view_get_model(GTK_TREE_VIEW(
                                                         W(apListView)));
 
-    if (!W(changed) && !W(cmd.changed))
+    if (!W(changed))
         ccreturn("NO SAVE is required");
 
-    /* Use-Case: commands has been updated:
-     *   - No other AP has not been selected
-     *   - Save/Quit button has been pressed after changes
-     */
-//   if (W(cmd.changed) && !W(changed))
-    // TODO: not optimal, research
-    if (W(cmd.changed))
-        update_ap_model_commands(app_data, W(ap.currentIter));
-
     cfg.version = g_strdup(PROGRAM_VERSION);
     cfg.access_points = get_ap_gslist_from_model(ap_model);
 
@@ -661,7 +661,6 @@ static void apply_changes(GtkWidget* widget, gpointer data)
     DBG("*** SAVE CONFIGURATION FILE ***");
 
     W(changed) = FALSE;
-    W(cmd.changed) = FALSE;
 }
 
 
