@@ -27,9 +27,11 @@
 #include "cellrendererpixbuf.h"
 
 #include <gtk/gtk.h>
+#include <glib.h>
 
 #ifdef MAEMO
   #include <hildon/hildon.h>
+  #include <conic.h>
 #endif
 
 // TODO: wrap to translation macroses
@@ -47,6 +49,9 @@ static void create_cmd_toolbar(GtkWidget*, AppData_s*);
 static void create_main_toolbar(GtkWidget*, AppData_s*);
 static void init_stock_icons();
 
+static void fill_avail_iaps_tree_model(GtkTreeModel*, gchar*);
+static void fill_avail_cmds_tree_model(GtkTreeModel*, GSList*);
+
 
 AppData_s* new_app_data()
 {
@@ -54,7 +59,6 @@ AppData_s* new_app_data()
 
     W(ap.currentIter) = NULL;
     W(changed) = FALSE;
-    W(cmd.changed) = FALSE;
 #ifdef MAEMO
     W(fullscreen) = FALSE;
 #endif
@@ -109,7 +113,7 @@ void create_main_view(AppData_s* app_data)
     gtk_frame_set_shadow_type(GTK_FRAME(left_frame), GTK_SHADOW_IN);
     gtk_frame_set_shadow_type(GTK_FRAME(right_frame), GTK_SHADOW_IN);
     /* Set initial size for the left frame */
-    gtk_widget_set_size_request(left_frame, 125, -1);
+    gtk_widget_set_size_request(left_frame, 150, -1);
 
     /* AP List */
     W(apListView) = gtk_tree_view_new();
@@ -256,8 +260,10 @@ static void init_stock_icons()
     static GtkStockItem stock_items[] = {
 	{ ENB_SUPERUSER_ENV_STOCK, NULL },
 	{ DSB_SUPERUSER_ENV_STOCK, NULL },
-	{ ENB_COMMAND_STOCK, NULL },
-	{ DSB_COMMAND_STOCK, NULL }
+	{ ENB_BG_COMMAND_STOCK, NULL },
+	{ DSB_BG_COMMAND_STOCK, NULL },
+	{ ENB_FG_COMMAND_STOCK, NULL },
+	{ DSB_FG_COMMAND_STOCK, NULL }
     };
 
     GtkIconFactory* icon_factory = gtk_icon_factory_new();
@@ -340,6 +346,8 @@ void create_ap_list_view(AppData_s* app_data)
     list_store = gtk_list_store_new(1, ACCESS_POINT_TYPE);
     gtk_tree_view_set_model(GTK_TREE_VIEW(W(apListView)),
                             GTK_TREE_MODEL(list_store));
+    /* Enable Drag'n'Drop */
+    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(W(apListView)), TRUE);
 
     /* Forbid to make a zero selection */
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(W(apListView)));
@@ -386,10 +394,16 @@ static void active_cmd_cell_func(GtkTreeViewColumn* column,
                                  gpointer data)
 {
     Command_s* cmd;
+    gchar* enabled_cmd_icon;
+    gchar* disabled_cmd_icon;
 
     gtk_tree_model_get(model, iter, 0, &cmd, -1);
+    enabled_cmd_icon = cmd->sync_exec ? ENB_FG_COMMAND_STOCK :
+	                                ENB_BG_COMMAND_STOCK;
+    disabled_cmd_icon = cmd->sync_exec ? DSB_FG_COMMAND_STOCK :
+	                                 DSB_BG_COMMAND_STOCK;
     g_object_set(cell, "stock-id", cmd->disabled ?
-		 DSB_COMMAND_STOCK : ENB_COMMAND_STOCK, NULL);
+		 disabled_cmd_icon : enabled_cmd_icon, NULL);
 
     free_command(cmd);
 }
@@ -446,6 +460,8 @@ void create_cmd_list_view(AppData_s* app_data)
     list_store = gtk_list_store_new(1, COMMAND_TYPE);
     gtk_tree_view_set_model(GTK_TREE_VIEW(W(commandListView)),
                             GTK_TREE_MODEL(list_store));
+    /* Enable Drag'n'Drop */
+    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(W(commandListView)), TRUE);
 
     g_object_unref(list_store);
 }
@@ -551,20 +567,28 @@ void select_treeview_row(GtkWidget* tree_view, GtkTreeIter* iter)
     gtk_tree_path_free(path);
 }
 
+
 gboolean show_add_ap_dialog(GtkWindow* parent,
 			    const gboolean edit,
 			    AccessPoint_s** ap)
 {
     GtkWidget* dialog;
-    GtkWidget* input_hbox = gtk_hbox_new(FALSE, 0);
+    GtkWidget* input_hbox = gtk_hbox_new(FALSE, 3);
     GtkWidget* label = gtk_label_new_with_mnemonic("_Name:");
-    GtkWidget* input_entry = gtk_entry_new();
+    GtkListStore* iaps_list_store = gtk_list_store_new(1, G_TYPE_STRING);
+    GtkWidget* input_entry = gtk_combo_box_entry_new_with_model(
+	                                GTK_TREE_MODEL(iaps_list_store), 0);
     gboolean disabled = edit ? (*ap)->disabled : FALSE;
     GtkWidget* disabled_check_button = make_check_button(
 	                                        "_Disabled",
 						disabled);
+
+    fill_avail_iaps_tree_model(GTK_TREE_MODEL(iaps_list_store),
+			       edit ? (*ap)->name : NULL);
+    g_object_unref(iaps_list_store);
+
     if (edit)
-	gtk_entry_set_text(GTK_ENTRY(input_entry), (*ap)->name);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(input_entry), 0);
 
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), input_entry);
     gtk_box_pack_start(GTK_BOX(input_hbox), label, FALSE, FALSE, 0);
@@ -580,7 +604,8 @@ gboolean show_add_ap_dialog(GtkWindow* parent,
 
     gtk_widget_show_all(dialog);
     gint response = gtk_dialog_run(GTK_DIALOG(dialog));
-    gchar* ap_name = (gchar *)gtk_entry_get_text(GTK_ENTRY(input_entry));
+    gchar* ap_name = (gchar *)gtk_combo_box_get_active_text(
+	                                  GTK_COMBO_BOX(input_entry));
     if (GTK_RESPONSE_OK != response || !g_utf8_strlen(ap_name, -1)) {
 	gtk_widget_destroy(dialog);
 	return FALSE;
@@ -589,7 +614,7 @@ gboolean show_add_ap_dialog(GtkWindow* parent,
     if (!edit)
 	*ap = g_new(AccessPoint_s, 1);
 
-    (*ap)->name = g_strdup(ap_name);
+    (*ap)->name = ap_name;
     (*ap)->disabled = gtk_toggle_button_get_active(
                     	         GTK_TOGGLE_BUTTON(disabled_check_button));
     if (!edit)
@@ -600,44 +625,138 @@ gboolean show_add_ap_dialog(GtkWindow* parent,
     return TRUE;
 }
 
+void fill_avail_iaps_tree_model(GtkTreeModel* model, gchar* top_iap)
+{
+#ifndef MAEMO
+    if (top_iap) {
+	GtkTreeIter iter;
 
+	gtk_list_store_insert(GTK_LIST_STORE(model), &iter, 0);
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, top_iap, -1);
+    }
+#else
+    GSList* iap_list;
+    gboolean top_iap_is_avail = !top_iap ? TRUE : FALSE;
+
+    iap_list = con_ic_connection_get_all_iaps(NULL);
+
+    for (GSList* iter = iap_list; iter; iter = g_slist_next(iter)) {
+	ConIcIap* iap = (ConIcIap *)(iter->data);
+	GtkTreeIter ap_iter;
+	const gchar* iap_name;
+
+	if (g_ascii_strcasecmp(con_ic_iap_get_bearer_type(iap),
+			       WLAN_BEARER_TYPE))
+	    continue;
+
+	iap_name = con_ic_iap_get_name(iap);
+	if (!top_iap_is_avail && !g_ascii_strcasecmp(iap_name, top_iap)) {
+	    top_iap_is_avail = TRUE;
+	    gtk_list_store_insert(GTK_LIST_STORE(model), &ap_iter, 0);
+	}
+	else
+	    gtk_list_store_append(GTK_LIST_STORE(model), &ap_iter);
+
+	gtk_list_store_set(GTK_LIST_STORE(model), &ap_iter, 0, iap_name, -1);
+    }
+
+    if (!top_iap_is_avail) {
+	GtkTreeIter iter;
+
+	gtk_list_store_insert(GTK_LIST_STORE(model), &iter, 0);
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, top_iap, -1);
+    }
+
+    if (iap_list) {
+	g_slist_foreach(iap_list, (GFunc)g_object_unref, NULL);
+	g_slist_free(iap_list);
+    }
+#endif
+}
+
+// TODO: very long, split up to small ones?
+// TODO: pass cmd_list instead AP model
 gboolean show_add_command_dialog(GtkWindow* parent,
+				 GtkTreeModel* ap_model,
 				 const gboolean edit,
 				 Command_s** cmd)
 {
     GtkWidget* dialog;
-    GtkWidget* input_hbox = gtk_hbox_new(FALSE, 0);
+    GtkWidget* tab_container = gtk_notebook_new();
+    GtkWidget* input_hbox = gtk_hbox_new(FALSE, 3);
+    GtkWidget* general_vbox  = gtk_vbox_new(TRUE, 10);
+    GtkWidget* advanced_vbox = gtk_vbox_new(TRUE, 10);
     GtkWidget* label = gtk_label_new_with_mnemonic("Co_mmand:");
     GtkWidget* input_entry = gtk_entry_new();
     gboolean disabled = edit ? (*cmd)->disabled : FALSE;
     gboolean superuser_env = edit ? (*cmd)->superuser_env : FALSE;
+    gboolean run_async = edit ? !(*cmd)->sync_exec : TRUE;
     GtkWidget* disabled_check_button = make_check_button(
 	                                        "_Disabled",
 						disabled);
     GtkWidget* superuser_env_check_button = make_check_button(
-	                                        "_Run as superuser",
+	                                        "Run as _superuser",
 						superuser_env);
+    GtkWidget* run_async_check_button = make_check_button(
+	                                        "Run in _background",
+						run_async);
+    /* Completion */
+    GtkEntryCompletion* completion_input_entry = gtk_entry_completion_new();
+    GtkListStore* cmds_list_store = gtk_list_store_new(1, G_TYPE_STRING);
+    GSList* ap_list = get_ap_gslist_from_model(ap_model);
+
+    fill_avail_cmds_tree_model(GTK_TREE_MODEL(cmds_list_store), ap_list);
+
+    g_slist_foreach(ap_list, (GFunc)free_access_point, NULL);
+    g_slist_free(ap_list);
 
     if (edit)
 	gtk_entry_set_text(GTK_ENTRY(input_entry), (*cmd)->command);
 
+    /* Setup entry completion */
+    gtk_entry_set_completion(GTK_ENTRY(input_entry), completion_input_entry);
+    gtk_entry_completion_set_model(completion_input_entry,
+				   GTK_TREE_MODEL(cmds_list_store));
+    gtk_entry_completion_set_text_column(completion_input_entry, 0);
+    g_object_unref(cmds_list_store);
+    g_object_unref(completion_input_entry);
+
+    /* Input */
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), input_entry);
     gtk_box_pack_start(GTK_BOX(input_hbox), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(input_hbox), input_entry, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(general_vbox), input_hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(general_vbox), disabled_check_button, FALSE,
+		                                               FALSE, 0);
+    /* Advanced */
+    gtk_box_pack_start(GTK_BOX(advanced_vbox), superuser_env_check_button,
+		                                          FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(advanced_vbox), run_async_check_button,
+		                                          FALSE, FALSE, 0);
 
-    dialog = show_add_or_edit_entity_basic_dialog(parent,
-					  edit ? "Edit Command" : "New Command",
-					  edit ? "_Edit" : "_Add",
-					  "_Cancel",
-				          input_hbox,
-				          disabled_check_button,
-					  superuser_env_check_button,
-				          NULL);
+    dialog = gtk_dialog_new_with_buttons(
+	                      edit ? "Edit Command" : "New Command",
+			      parent,
+                              GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			      edit ? "_Edit" : "_Add", GTK_RESPONSE_OK,
+			      "_Cancel", GTK_RESPONSE_CANCEL, NULL);
+
+    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(tab_container),
+			     general_vbox,
+			     make_label_with_markup("General"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(tab_container),
+			     advanced_vbox,
+			     make_label_with_markup("Advanced"));
+    gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
+				tab_container);
 
     /* Extend dialog's width to extend input_hbox's width */
-    resize_window_by_delta(GTK_WINDOW(dialog), 175, 0);
+    resize_window_by_delta(GTK_WINDOW(dialog), 200, 0);
 
     gtk_widget_show_all(dialog);
+    gtk_widget_grab_focus(input_entry);
     gint response = gtk_dialog_run(GTK_DIALOG(dialog));
     gchar* command = (gchar *)gtk_entry_get_text(GTK_ENTRY(input_entry));
     if (GTK_RESPONSE_OK != response || !g_utf8_strlen(command, -1)) {
@@ -653,11 +772,47 @@ gboolean show_add_command_dialog(GtkWindow* parent,
                     	         GTK_TOGGLE_BUTTON(disabled_check_button));
     (*cmd)->superuser_env = gtk_toggle_button_get_active(
                   	         GTK_TOGGLE_BUTTON(superuser_env_check_button));
+    (*cmd)->sync_exec = !gtk_toggle_button_get_active(
+	                         GTK_TOGGLE_BUTTON(run_async_check_button));
 
     gtk_widget_destroy(dialog);
 
     return TRUE;
 }
 
+void fill_avail_cmds_tree_model(GtkTreeModel* model, GSList* ap_list)
+{
+    GHashTable* cmd_htable = g_hash_table_new_full(g_str_hash, g_str_equal,
+						   (GDestroyNotify)g_free,
+						   NULL);
+
+    gtk_list_store_clear(GTK_LIST_STORE(model));
+
+    /* Pass all avail commands to hash table to collect only uniq */
+    for (GSList* ap_iter = ap_list; ap_iter; ap_iter = g_slist_next(ap_iter)) {
+	AccessPoint_s* ap = AP_LIST_CAST(ap_iter);
+
+	for (GSList* cmd_iter = ap->commands; cmd_iter;
+	                                    cmd_iter = g_slist_next(cmd_iter)) {
+	    Command_s* cmd = CMD_LIST_CAST(cmd_iter);
+
+	    g_hash_table_insert(cmd_htable, g_strdup(cmd->command), 0);
+	}
+    }
+
+    GList* uniq_commands = g_hash_table_get_keys(cmd_htable);
+    for (GList* cmd_iter = uniq_commands;
+	 cmd_iter; cmd_iter = g_list_next(cmd_iter)) {
+	gchar* command = (gchar *)(cmd_iter->data);
+	GtkTreeIter iter;
+
+	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, command, -1);
+    }
+
+    g_list_free(uniq_commands);
+    g_hash_table_destroy(cmd_htable);
+}
+
 
 
